//******************************************************************************
//
//	File:			BitCheck.cpp
//
//	Description:	Makes sure all bits in a file are 0 or 1
//	
//	License:		Freeware
//	
//	Written by:		Mathias Agopian
//
//******************************************************************************

#include <assert.h>
#include <stdio.h>
#include <Errors.h>
#include <File.h>


status_t isbitvalid(char *bytePtr, uint8 bit, uint64 *bit_value)
{
	if (bit >= 8) return B_BAD_VALUE;
	*bit_value = ((*bytePtr) & (1<<bit)) >> bit;
	if ((*bit_value != 0) && (*bit_value != 1))
		return B_ERROR;
	return B_OK;
}

int main(int argc, char **argv)
{
	assert(sizeof(char));

	if (argc != 2) {
		printf("usage: %s <filename>\n", argv[0]);
		return 0;
	}

	BFile file(argv[1], B_READ_ONLY);
	char byte;
	uint64 bit_value;
	ssize_t offset = 0;
	do {
		ssize_t r = file.Read(&byte, 1);
		if (r == 1) {
			for (uint8 i=0 ; i<8 ; i++) {
				if (isbitvalid(&byte, i, &bit_value) != B_OK) {
					printf("Bit %d at offset %ld has a value of %Lu\n", i, offset, bit_value);
				}
			}
			offset++;
			continue;
		}
		break;
	} while(true);
	printf("Tested %lu bits. Your bits are all okay.\n", (unsigned int)offset*8);
	return 0;
}
